/*
 * JodaFriday13th - Searching Friday the 13th dates with Joda-Time
 * Copyright (C) 2009 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.friday13th;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.joda.time.DateTime;
import org.joda.time.DateTimeConstants;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

/**
 * Searching Friday the 13th dates with Joda-Time
 * 
 * @author Christian Schenk
 */
public class Friday13th {

	/**
	 * Gets the next <em>n</em> days of Friday the 13th.
	 */
	public static List<DateTime> getNextNthFriday13th(final int n) {
		final List<DateTime> fridays = new ArrayList<DateTime>();

		DateTime dt = new DateTime();
		int curN = 0;
		while (curN < n) {
			dt = dt.plusDays(1);

			if (dt.getDayOfMonth() != 13) continue;
			if (dt.getDayOfWeek() != DateTimeConstants.FRIDAY) continue;

			fridays.add(new DateTime(dt.toInstant()));
			curN++;
		}

		return fridays;
	}

	/**
	 * Returns the next year with the most days of Friday the 13th dates.
	 */
	public static YearCount getYearWithLotsOfFriday13th() {
		final int MAX_YEARS = 20;

		final Map<Integer, Integer> yearFriday13thCount = new HashMap<Integer, Integer>();

		// get fridays and add them to the map
		final List<DateTime> friday13ths = getNextNthFriday13th(MAX_YEARS);
		for (final DateTime friday : friday13ths) {
			final int year = friday.getYear();
			if (yearFriday13thCount.get(year) == null) yearFriday13thCount.put(year, 0);
			yearFriday13thCount.put(year, yearFriday13thCount.get(year) + 1);
		}

		// find year with the most friday 13th
		int year = Integer.MAX_VALUE, count = Integer.MIN_VALUE;
		for (final int curYear : yearFriday13thCount.keySet()) {
			final int curCount = yearFriday13thCount.get(curYear);
			if (curCount > count && curYear < year) {
				year = curYear;
				count = curCount;
			}
		}

		final int theYear = year;
		final int theCount = count;

		return new YearCount() {
			public int getCount() {
				return theCount;
			}

			public int getYear() {
				return theYear;
			}
		};
	}

	/**
	 * Encapsulates a year and its count of Friday the 13th dates.
	 */
	private interface YearCount {
		public int getYear();

		public int getCount();
	}

	public static void main(final String[] args) {
		final List<DateTime> fridays = getNextNthFriday13th(3);
		final DateTimeFormatter format = DateTimeFormat.forPattern("yyyy-MM-dd");
		System.out.println("Next Friday the 13th dates:");
		for (final DateTime friday : fridays) {
			System.out.println(format.print(friday));
		}
		System.out.println("");

		final YearCount yc = getYearWithLotsOfFriday13th();
		System.out.println("The year " + yc.getYear() + " has got " + yc.getCount() + " Friday the 13th dates.");
	}
}